==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webdav/inc/webdav.php <==
<?php
include_once 'datastore.php';
include_once 'xml.php';

//************************************************************
// Hlavn tda realizujc WebDAV protokol
// Main class implementing WebDAV protocol
//************************************************************

class webdav_session
{
  var $_datastore;
  var $_authenticated;
  
  var $_server;
  var $_email;
  var $_folder;
  var $_local_id;
  var $_depth;
  var $_body;

//************************************************************
    
  function webdav_session()
  {
    $this->_datastore=new datastore();
  }  

//************************************************************
    
  function parse_uri()
  {
    $request_uri=$_SERVER['REQUEST_URI'];
    $result=array();
    
    $fields=explode('/',$request_uri);
    
    if ($request_uri[strlen($request_uri)-1]!='/')
      $result['local_id']=$fields[count($fields)-1];
        
    unset($fields[count($fields)-1]);
    unset($fields[0]);
    unset($fields[1]);
    
    if (strpos($fields[2],'~')===0)
		{
			$result['email']=substr($fields[2],1);
      unset($fields[2]);
    }
    
    $result['folder']=implode("\\",$fields);
    
    return $result;        
  }

//************************************************************
  
  function read_request()
  {
    $uri=$this->parse_uri();

		$host=$_SERVER['HTTP_HOST'];
    if ($pos=strpos($host,":"))
      $host=substr($host,0,$pos);
    $this->_server=($_SERVER['HTTPS']=='ON'?'https://':'http://').$host.($_SERVER['SERVER_PORT']==80?'':(':'.$_SERVER['SERVER_PORT'])).'/webdav'.($uri['email']?('/~'.$uri['email']):'');
    
    $this->_email=$uri['email'];
    $this->_folder=$uri['folder'];
    $this->_local_id=($uri['local_id']!='null')?$uri['local_id']:'';
    $this->_depth=$_SERVER[HTTP_DEPTH];   

    if ($this->_depth)
			$log_headers .= "Depth: $this->_depth ";

    $log_str='<<< '.$_SERVER['REQUEST_METHOD'].' '.$_SERVER['REQUEST_URI'].($log_headers?' ['.trim($log_headers).']':'')."\r\n";
     
    $this->_body=file_get_contents("php://input");
    
    $log_str.=$this->_body."\r\n";
    
    do_log($log_str,2);
  }
  
  function user_logged_on()
  {
    if ($this->_authenticated)
      return $this->_datastore->user_logged_on();
    else
      return true;  
  }

//************************************************************
  
  function check_authorization()
  {
	  if (!$this->_authenticated)
    {
      $auth=$_SERVER['HTTP_AUTHORIZATION'];
      $p=strpos($auth,' ');
      $auth=substr($auth,$p+1,strlen($auth)-$p-1);
      
      $auth=base64_decode($auth);
      $p=strpos($auth,':');
      
      $username=str_replace('%','@',substr($auth,0,$p));
      $password=substr($auth,$p+1,strlen($auth)-$p-1);
      
      $this->_authenticated=$this->_datastore->login($username,$password,$this->_email);
    }
    return $this->_authenticated;  
  } 

//************************************************************
  
  function http_response($status,$headers=array(),$body='')
  {
    header(($_SERVER['HTTPS']=='ON'?'HTTPS':'HTTP').'/1.1 '.$status);
   
    foreach($headers as $header)
    {
      header($header);
      $log_headers.=$header.' ';
    }
    
    header('Allow: MKCALENDAR, MOVE, DELETE, PUT, PROPFIND, REPORT, GET');
    header('DAV: 1, calendar-access');

    // Do not sent content-length - we are running in gzip mode
    if (!$body)
			header('Content-Length: '.strlen($body));
   
    $log_str=">>> ".($_SERVER['HTTPS']=='ON'?'HTTPS':'HTTP').'/1.1 '.$status.($log_headers?' ['.trim($log_headers).']':'')."\r\n";

    if ($body)
    {
      // Start gzip output handler
      if (!ini_get("output_handler"))    
        ob_start("ob_gzhandler");

      echo $body;
      $log_str.=$body;
    }
    
    do_log($log_str,2);  
  }

//************************************************************
    
  function handle_request()
  {
    $method=strtolower($_SERVER['REQUEST_METHOD']);
    $handler='handle_'.$method;
    
    $this->read_request();
     
    if (!$this->check_authorization() && $method!='options')
    {
      $headers=array('WWW-Authenticate: Basic realm="'.$_SERVER['HTTP_HOST'].'"');
      $this->http_response('401 Unauthorized',$headers);
      return;
    }    
    
    if (method_exists($this,$handler)) 
      $this->$handler();
    else
      $this->http_response('405 Method not allowed');
  }  

//************************************************************
  
  function handle_options() 
  {
    foreach(get_class_methods($this) as $method) 
      if (!strncmp('handle_',$method,7) && $method!='handle_request') 
      {
        $method=strtoupper(substr($method,7));
        $allow[$method]=$method;          
      }
      
    $headers=array('DAV: 1, calendar-access','Allow: '.join(', ',$allow));
    $this->http_response('200 OK',$headers);    
  }
}
?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webdav/inc/webdav.php <==
<?php
include_once 'datastore.php';
include_once 'xml.php';

//************************************************************
// Hlavn tda realizujc WebDAV protokol
// Main class implementing WebDAV protocol
//************************************************************

class webdav_session
{
  var $_datastore;
  var $_authenticated;
  
  var $_server;
  var $_server_root;
  var $_users;
  var $_email;
  var $_folder;
  var $_local_id;
  var $_depth;
  var $_body;

//************************************************************
    
  function webdav_session()
  {
    $this->_datastore=new datastore();
  }  

//************************************************************
    
  function parse_uri()
  {
    $request_uri=$_SERVER['REQUEST_URI'];
    $result=array();
    
    $fields=explode('/',$request_uri);

    if ($request_uri[strlen($request_uri)-1]!='/')
      $result['local_id']=$fields[count($fields)-1];
      
		if ($fields[2]=='users')
		{
			$result['email']=$fields[3];
			$result['users']=true;
		}
		else
		{
    	$result['email']=$fields[2];
    	$result['users']=false;
    
    	unset($fields[count($fields)-1]);
    	unset($fields[0]);
    	unset($fields[1]);
    	unset($fields[2]);
    
      $result['folder']=implode("\\",$fields);
		}
		
    return $result;        
  }

//************************************************************
  
  function read_request()
  {
    $uri=$this->parse_uri();

		$host=$_SERVER['HTTP_HOST'];
    if ($pos=strpos($host,":"))
      $host=substr($host,0,$pos);

		$port=($_SERVER['HTTPS']=='ON')?(($_SERVER['SERVER_PORT']==443)?'':(':'.$_SERVER['SERVER_PORT'])):(($_SERVER['SERVER_PORT']==80)?'':(':'.$_SERVER['SERVER_PORT']));
		$this->_server_root=($_SERVER['HTTPS']=='ON'?'https://':'http://').$host.$port.'/webdav/';
		$this->_server=$this->_server_root.($uri['users']?'users/':'');
    
		$this->_users=$uri['users'];
		$this->_email=$uri['email'];
    $this->_folder=$uri['folder'];
    $this->_local_id=($uri['local_id']!='null')?$uri['local_id']:'';
    $this->_depth=isset($_SERVER[HTTP_DEPTH])?$_SERVER[HTTP_DEPTH]:'infinity';

    if ($this->_depth)
			$log_headers .= "Depth: $this->_depth ";

    $log_str='<<< '.$_SERVER['REQUEST_METHOD'].' '.$_SERVER['REQUEST_URI'].($log_headers?' ['.trim($log_headers).']':'')."\r\n";
     
    $this->_body=file_get_contents("php://input");
    
    $log_str.=$this->_body."\r\n";
    
    do_log($log_str,2);
  }
  
  function user_logged_on()
  {
    if ($this->_authenticated)
      return $this->_datastore->user_logged_on();
    else
      return true;  
  }

//************************************************************
  
  function check_authorization()
  {
	  if (!$this->_authenticated)
    {
      $auth=$_SERVER['HTTP_AUTHORIZATION'];
      $p=strpos($auth,' ');
      $auth=substr($auth,$p+1,strlen($auth)-$p-1);
      
      $auth=base64_decode($auth);
      $p=strpos($auth,':');
      
      $username=str_replace('%','@',substr($auth,0,$p));
      $password=substr($auth,$p+1,strlen($auth)-$p-1);
      
      $this->_authenticated=$this->_datastore->login($username,$password,$this->_email);
    }
    return $this->_authenticated;  
  } 

//************************************************************
  
  function http_response($status,$headers=array(),$body='')
  {
    header(($_SERVER['HTTPS']=='ON'?'HTTPS':'HTTP').'/1.1 '.$status);
   
    foreach($headers as $header)
    {
      header($header);
      $log_headers.=$header.' ';
    }
    
    header('Allow: MKCALENDAR, MOVE, DELETE, PUT, PROPFIND, REPORT, GET');
		//header('DAV: 1, calendar-access');
		header('DAV: 1, access-control, calendar-access, calendar-schedule, calendar-availability, inbox-availability, calendar-proxy');

    // Do not sent content-length - we are running in gzip mode
    if (!$body)
			header('Content-Length: '.strlen($body));
   
    $log_str=">>> ".($_SERVER['HTTPS']=='ON'?'HTTPS':'HTTP').'/1.1 '.$status.($log_headers?' ['.trim($log_headers).']':'')."\r\n";

    if ($body)
    {
      // Start gzip output handler
      if (!ini_get("output_handler"))    
        ob_start("ob_gzhandler");

      echo $body;
      $log_str.=$body;
    }
    
    do_log($log_str,2);  
  }

//************************************************************
    
  function handle_request()
  {
    $method=strtolower($_SERVER['REQUEST_METHOD']);
    $handler='handle_'.$method;
    
    $this->read_request();
     
    if (!$this->check_authorization() && $method!='options')
    {
      $headers=array('WWW-Authenticate: Basic realm="'.$_SERVER['HTTP_HOST'].'"');
     
			$this->http_response('401 Unauthorized',$headers);
      return;
    }    
    
    if (method_exists($this,$handler)) 
      $this->$handler();
    else
      $this->http_response('405 Method not allowed');
  }  

//************************************************************
  
  function handle_options() 
  {
    foreach(get_class_methods($this) as $method) 
      if (!strncmp('handle_',$method,7) && $method!='handle_request') 
      {
        $method=strtoupper(substr($method,7));
        $allow[$method]=$method;          
      }
      
    $headers=array('DAV: 1, calendar-access','Allow: '.join(', ',$allow));
    $this->http_response('200 OK',$headers);    
  }
}
?>
